@@ -25,6 +25,7 @@ from cryptoparser.common.parse import (
     ParserText,
 )
 from cryptoparser.common.exception import NotEnoughData, TooMuchData, InvalidValue, InvalidType
+from cryptoparser.common.utils import bytes_to_hex_string
 
 
 def _default(
@@ -41,13 +42,29 @@ json.JSONEncoder.default = _default
 
 class Serializable(object):  # pylint: disable=too-few-public-methods
     @staticmethod
-    def _get_ordered_dict(dict_value):
+    def _filter_out_non_human_friendly(obj, dict_value, human_friendly_only):
+        if not attr.has(type(obj)) or not human_friendly_only:
+            return dict_value
+
+        fields_dict = attr.fields_dict(type(obj))
+        dict_value = OrderedDict([
+            (name, value)
+            for name, value in dict_value.items()
+            if name not in fields_dict or fields_dict[name].metadata.get('human_friendly', True)
+        ])
+
+        return dict_value
+
+    @staticmethod
+    def _get_ordered_dict(dict_value, human_friendly_only=False):
         if attr.has(type(dict_value)):
+            obj = dict_value
             dict_value = OrderedDict([
                 (name, getattr(dict_value, name))
                 for name, field in attr.fields_dict(type(dict_value)).items()
                 if not name.startswith('_')
             ])
+            dict_value = Serializable._filter_out_non_human_friendly(obj, dict_value, human_friendly_only)
             keys = dict_value.keys()
         elif isinstance(dict_value, OrderedDict):
             keys = dict_value.keys()
@@ -73,8 +90,10 @@ class Serializable(object):  # pylint: disable=too-few-public-methods
             result = {obj.name: obj.value}
         elif isinstance(obj, six.string_types + six.integer_types + (float, bool, )) or obj is None:
             result = obj
+        elif isinstance(obj, (bytes, bytearray)):
+            result = bytes_to_hex_string(obj, separator=':', lowercase=False)
         else:
-            result = repr(obj)
+            result = str(obj)
 
         return result
 
@@ -126,8 +145,9 @@ class Serializable(object):  # pylint: disable=too-few-public-methods
 
         if hasattr(obj, '_asdict'):
             dict_value = obj._asdict()
+            dict_value = Serializable._filter_out_non_human_friendly(obj, dict_value, human_friendly_only=True)
         else:
-            dict_value = Serializable._get_ordered_dict(obj)
+            dict_value = Serializable._get_ordered_dict(obj, human_friendly_only=True)
 
         result = ''
         name_dict = cls._markdown_human_readable_names(obj, dict_value)
@@ -187,6 +207,8 @@ class Serializable(object):  # pylint: disable=too-few-public-methods
             result = cls._markdown_result_complex(obj, level)
         elif isinstance(obj, (list, tuple, set, ArrayBase)):
             result = cls._markdown_result_list(obj, level)
+        elif isinstance(obj, (bytes, bytearray)):
+            result = False, bytes_to_hex_string(obj, separator=':', lowercase=False)
         else:
             result = False, str(obj)
 
@@ -463,7 +485,7 @@ class VectorString(ArrayBase):
         vector_param = self.get_param()
 
         body_composer = ComposerText(vector_param.encoding)
-        body_composer.compose_parsable_array(self._items, vector_param.separator)
+        body_composer.compose_parsable_array(self._items, vector_param.separator, vector_param.fallback_class)
 
         header_composer = ComposerBinary()
         header_composer.compose_numeric(body_composer.composed_length, self.param.item_num_size)
@@ -623,6 +645,17 @@ class ThreeByteEnumParsable(NByteEnumParsable):
         raise NotImplementedError()
 
 
+class FourByteEnumParsable(NByteEnumParsable):
+    @classmethod
+    def get_byte_num(cls):
+        return 4
+
+    @classmethod
+    @abc.abstractmethod
+    def get_enum_class(cls):
+        raise NotImplementedError()
+
+
 class NByteEnumComposer(enum.Enum):
     def __repr__(self):
         return self.__class__.__name__ + '.' + self.name
@@ -661,6 +694,12 @@ class ThreeByteEnumComposer(NByteEnumComposer):
         return 3
 
 
+class FourByteEnumComposer(NByteEnumComposer):
+    @classmethod
+    def get_byte_num(cls):
+        return 4
+
+
 class StringEnumParsableBase(ParsableBaseNoABC):
     @classmethod
     @abc.abstractmethod
@@ -677,7 +716,7 @@ class StringEnumParsableBase(ParsableBaseNoABC):
         enum_items.sort(key=lambda color: len(color.value.code), reverse=True)
 
         try:
-            code = parsable.decode('ascii')
+            code = six.ensure_text(parsable, 'ascii')
         except UnicodeDecodeError as e:
             six.raise_from(InvalidValue(parsable, cls), e)
 
@@ -688,7 +727,7 @@ class StringEnumParsableBase(ParsableBaseNoABC):
         raise InvalidValue(parsable, cls, 'code')
 
     def compose(self):
-        return self._asdict().encode('ascii')
+        return six.ensure_binary(self._asdict(), 'ascii')
 
     def _asdict(self):
         return getattr(self, 'value').code
@@ -783,7 +822,10 @@ class OpaqueEnumParsable(Vector):
     @classmethod
     def _parse(cls, parsable):
         opaque, parsed_length = super(OpaqueEnumParsable, cls)._parse(parsable)
-        code = bytearray(opaque).decode(cls.get_encoding())
+        code = six.ensure_text(
+            b''.join([six.int2byte(opaque_item) for opaque_item in opaque]),
+            cls.get_encoding()
+        )
 
         try:
             parsed_object = next(iter([
@@ -792,7 +834,7 @@ class OpaqueEnumParsable(Vector):
                 if enum_item.value.code == code
             ]))
         except StopIteration as e:
-            six.raise_from(InvalidValue(code.encode(cls.get_encoding()), cls), e)
+            six.raise_from(InvalidValue(code, cls), e)
 
         return parsed_object, parsed_length
 
@@ -812,7 +854,7 @@ class OpaqueEnumComposer(enum.Enum):
 
     def compose(self):
         composer = ComposerBinary()
-        value = self.value.code.encode(self.get_encoding())  # pylint: disable=no-member
+        value = six.ensure_binary(self.value.code, self.get_encoding())  # pylint: disable=no-member
 
         composer.compose_bytes(value, 1)
 
