@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
 import abc
-import copy
 import datetime
 import enum
 import struct
@@ -15,7 +14,7 @@ import dateutil
 import dateutil.parser
 import dateutil.tz
 
-from cryptoparser.common.exception import NotEnoughData, TooMuchData, InvalidValue
+from cryptoparser.common.exception import NotEnoughData, TooMuchData, InvalidType, InvalidValue
 import cryptoparser.common.utils
 
 
@@ -155,7 +154,7 @@ class ParserBase(collections_abc.Mapping):
 
         value = self._parsable[self._parsed_length:self._parsed_length + parsable_length]
         try:
-            value = value.decode(encoding)
+            value = six.ensure_text(value, encoding)
             if converter != str:
                 value = converter(value)
             self._parsed_values[name] = value
@@ -180,7 +179,7 @@ class ParserText(ParserBase):
             min_count,
             max_count
     ):
-        separators = separators.encode(self._encoding)
+        separators = six.ensure_binary(separators, self._encoding)
 
         count = 0
         actual_offset = count_offset
@@ -273,14 +272,14 @@ class ParserText(ParserBase):
             may_end):
         try:
             if not isinstance(item_class, type):
-                item = item_class(self._parsable[item_offset:item_end].decode(self._encoding))
+                item = item_class(six.ensure_text(self._parsable[item_offset:item_end], self._encoding))
             elif issubclass(item_class, ParsableBaseNoABC):
                 item, parsed_length = item_class.parse_immutable(self._parsable[item_offset:item_end])
                 item_end = item_offset + parsed_length
-            elif issubclass(item_class, str):
-                item = self._parsable[item_offset:item_end].decode(self._encoding)
+            elif issubclass(item_class, six.string_types):
+                item = six.ensure_text(self._parsable[item_offset:item_end], self._encoding)
             else:
-                item = item_class(self._parsable[item_offset:item_end].decode(self._encoding))
+                item = item_class(six.ensure_text(self._parsable[item_offset:item_end], self._encoding))
         except (InvalidValue, ValueError, UnicodeError) as e:
             if fallback_class is not None:
                 parsed_value, parsed_length = self._parse_string_until_separator(
@@ -304,7 +303,7 @@ class ParserText(ParserBase):
             separator_spaces=''
     ):
         item_end = None
-        byte_separators = [separator.encode(self._encoding) for separator in separators]
+        byte_separators = [six.ensure_binary(separator, self._encoding) for separator in separators]
 
         for separator_end in range(item_offset, len(self._parsable) + 1):
             for separator in byte_separators:
@@ -320,7 +319,7 @@ class ParserText(ParserBase):
             item_end = len(self._parsable)
 
         separator_space_count = 0
-        byte_separator_spaces = separator_spaces.encode(self._encoding)
+        byte_separator_spaces = six.ensure_binary(separator_spaces, self._encoding)
         while (item_end > item_offset and
                 self._parsable[
                     item_end - separator_space_count - 1:
@@ -427,7 +426,7 @@ class ParserText(ParserBase):
     def parse_date_time(self, name):
         try:
             value = self._parsable[self._parsed_length:]
-            date_time = dateutil.parser.parse(value.decode(self._encoding))
+            date_time = dateutil.parser.parse(six.ensure_text(value, self._encoding))
         except ValueError as e:
             six.raise_from(InvalidValue(value, type(self), 'value'), e)
 
@@ -457,7 +456,9 @@ class ParserBinary(ParserBase):
         if value[0] == 0xffffffffffffffff:
             self._parsed_values[name] = None
         else:
-            self._parsed_values[name] = datetime.datetime.utcfromtimestamp(0x00000000ffffffff & value[0])
+            self._parsed_values[name] = datetime.datetime.fromtimestamp(
+                0x00000000ffffffff & value[0], dateutil.tz.UTC
+            )
 
     def _parse_numeric_array(self, name, item_num, item_size, item_numeric_class):
         if self._parsed_length + (item_num * item_size) > len(self._parsable):
@@ -506,6 +507,35 @@ class ParserBinary(ParserBase):
         self._parsed_length += parsed_length
         self._parsed_values[name] = value
 
+    def parse_ssh_mpint(self, name):
+        if self.unparsed_length < 4:
+            raise NotEnoughData(bytes_needed=4 - self.unparsed_length)
+
+        mpint_length, parsed_length = self._parse_numeric_array(name, 1, 4, int)
+        mpint_length = mpint_length[0]
+
+        negative = (mpint_length and (six.indexbytes(self._parsable, self._parsed_length + 4) >= 0x80))
+
+        if mpint_length % 4:
+            pad_byte = six.int2byte(0xff) if negative else six.int2byte(0x00)
+            pad_bytes = (4 - (mpint_length % 4)) * pad_byte
+        else:
+            pad_bytes = b''
+
+        parsable = pad_bytes + self._parsable[self._parsed_length + 4:]
+        parser = ParserBinary(parsable)
+        parser.parse_numeric_array('mpint_part_array', (mpint_length + len(pad_bytes)) // 4, 4, int)
+
+        value = 0
+        for mpint_part in parser['mpint_part_array']:
+            value = (value << 32) + mpint_part
+        if negative:
+            complement = 1 << (8 * (mpint_length + len(pad_bytes)))
+            value -= complement
+
+        self._parsed_values[name] = value
+        self._parsed_length += parsed_length + mpint_length
+
     def _parse_bytes(self, size):
         if self.unparsed_length < size:
             raise NotEnoughData(bytes_needed=size - self.unparsed_length)
@@ -646,7 +676,7 @@ class ComposerBase(object):
         return len(self._composed)
 
     def _compose_string_array(self, values, encoding, separator):
-        separator = bytearray(separator.encode(encoding))
+        separator = bytearray(six.ensure_binary(separator, encoding))
         composed_str = bytearray()
 
         for value in values:
@@ -654,7 +684,7 @@ class ComposerBase(object):
                 if isinstance(value, ParsableBaseNoABC):
                     composed_str += value.compose()
                 else:
-                    composed_str += six.text_type(value).encode(encoding)
+                    composed_str += six.ensure_binary(six.text_type(value), encoding)
             except UnicodeError as e:
                 six.raise_from(InvalidValue(value, type(self)), e)
 
@@ -674,7 +704,7 @@ class ComposerText(ComposerBase):
         for value in values:
             composed_str += '{:d}{}'.format(value, separator)
 
-        self._composed += composed_str[:len(composed_str) - len(separator)].encode(self._encoding)
+        self._composed += six.ensure_binary(composed_str[:len(composed_str) - len(separator)], self._encoding)
 
     def compose_numeric(self, value):
         self._compose_numeric_array([value, ], separator='')
@@ -691,10 +721,21 @@ class ComposerText(ComposerBase):
     def compose_parsable(self, value):
         self._composed += value.compose()
 
-    def compose_parsable_array(self, values, separator=','):
-        separator = separator.encode(self._encoding)
+    def compose_parsable_array(self, values, separator=',', fallback_class=None):
+        separator = six.ensure_binary(separator, self._encoding)
 
-        self._composed += bytearray(separator).join(map(lambda item: item.compose(), values))
+        composed_items = []
+        for item in values:
+            if isinstance(item, (ComposerBase, ParsableBase, ParsableBaseNoABC)):
+                composed_item = item.compose()
+            elif fallback_class is not None and isinstance(item, fallback_class):
+                composed_item = six.ensure_binary(item, self._encoding)
+            else:
+                raise InvalidType()
+
+            composed_items.append(composed_item)
+
+        self._composed += bytearray(separator).join(composed_items)
 
     def compose_separator(self, value):
         self.compose_string(value)
@@ -714,8 +755,7 @@ class ComposerBinary(ComposerBase):
         if value is None:
             value = 0xffffffffffffffff
         else:
-            date_time = copy.copy(value)
-            value = int(time.mktime(date_time.replace(tzinfo=dateutil.tz.UTC).timetuple()))
+            value = int(time.mktime(value.timetuple())) - time.timezone
 
         return self._compose_numeric_array([value, ], 8)
 
@@ -749,6 +789,34 @@ class ComposerBinary(ComposerBase):
             flag |= value
         self._compose_numeric_array([flag, ], item_size)
 
+    def compose_ssh_mpint(self, value):
+        negative = value < 0
+        array_length = value.bit_length() // 32
+
+        if value.bit_length() % 32:
+            array_length += 1
+        if negative:
+            positive_value = (1 << ((value.bit_length() // 8 * 8) + 8)) + value
+        else:
+            positive_value = value
+
+        value_numeric_array = []
+        for mpint_offset in range(0, array_length * 32, 32):
+            value_numeric_array.append(positive_value >> mpint_offset & 0xffffffff)
+
+        composer = ComposerBinary()
+        composer.compose_numeric_array(reversed(value_numeric_array), 4)
+        value_bytes = composer.composed_bytes.lstrip(b'\x00')
+
+        if value_bytes and bool(value_bytes[0] & 0x80) != negative:
+            pad_byte = b'\xff' if negative else b'\x00'
+        else:
+            pad_byte = b''
+
+        self.compose_numeric(len(pad_byte) + len(value_bytes), 4)
+
+        self._composed += pad_byte + value_bytes
+
     def compose_parsable(self, value, item_size=None):
         composed = value.compose()
         if item_size is not None:
@@ -768,7 +836,7 @@ class ComposerBinary(ComposerBase):
 
     def compose_string(self, value, encoding, item_size):
         try:
-            value = value.encode(encoding)
+            value = six.ensure_binary(value, encoding)
         except UnicodeError as e:
             six.raise_from(InvalidValue(value, type(self)), e)
 
